<?php

namespace Drupal\book_pdfs\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;

class BooktreeController extends ControllerBase {

  /**
   * Generate the book tree.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The book to get the tree for.
   * @param bool $link_to_pdf_sections
   *   Whether to link to PDF sections instead of website URLs.
   *
   * @return array
   *   The render array.
   */
  public function tree($book, $link_to_pdf_sections = false) {
    $content = '';
    $booktree_start = $book->id();

    // Retrieve the maximum depth and maximum title length from the configuration.
    $maxricursione = \Drupal::config('book_pdfs.settings')->get('booktree_deep') + 2;
    $trimval = \Drupal::config('book_pdfs.settings')->get('booktree_trim');

    $node = \Drupal::entityTypeManager()->getStorage('node')->load($booktree_start);
    $node = \Drupal::service('entity.repository')->getTranslationFromContext($node);
    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();

    $cache_tags[] = "node:$booktree_start:$langcode";
    $cache_tags[] = "bid:" . $node->book['bid'] . ":$langcode";

    $content .= "<h1>" . $this->t('Table of contents') . "</h1>";
    $content .= $this->booktree_mostra_figli($node, $node, 1, $maxricursione, $trimval, $cache_tags, $link_to_pdf_sections);

    return [
      '#type' => 'markup',
      '#title' => t(':title', [':title' => $node->getTitle()]),
      '#markup' => $content,
      '#cache' => [
        'contexts' => ['url.path'],
        'tags' => $cache_tags
      ],
    ];
  }

  /**
   * Recursively generates the book tree structure.
   *
   * This function builds the list items for each node in the book structure, applying
   * different linking behaviors based on whether the links should lead to PDF sections
   * or website URLs.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The current node being processed.
   * @param \Drupal\node\Entity\Node $node_start
   *   The starting node of the book.
   * @param int $ricursione
   *   The current depth in the recursion.
   * @param int $maxricursione
   *   The maximum depth allowed for recursion.
   * @param string $trimval
   *   The maximum length for trimming titles.
   * @param array &$cache_tags
   *   An array of cache tags to be updated for cache invalidation.
   * @param bool $link_to_pdf_sections
   *   Flag indicating whether to link to sections within the PDF (true) or to web URLs (false).
   *
   * @return string
   *   The HTML content for the book tree for the given node.
   */
  private function booktree_mostra_figli($node, $node_start, $ricursione, $maxricursione, $trimval, &$cache_tags, $link_to_pdf_sections) {
    $content = $c = '';

    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    if ($ricursione < $maxricursione) {
      $sql = "SELECT DISTINCT b.nid, b.pid, b.weight
           FROM {book} as b
           inner join {node} as n ON n.nid = b.nid
           WHERE b.pid = :pid
           ORDER by b.weight";

      $sql_args = [':pid' => $node->id()];
      $query = \Drupal::database()->query($sql, $sql_args);
      $children = $query->fetchAll();

      if ($node->id() != $node_start->id()) {
        $node = \Drupal::service('entity.repository')->getTranslationFromContext($node);
        // Create link depending on settings
        if (0 && $link_to_pdf_sections) {
          $url = Url::fromRoute('<none>', [], ['fragment' => 'section-' . $node->id()]);
          $link = Link::fromTextAndUrl($node->label(), $url)->toString();
        }
        else {
          $link = $node->toLink()->toString();
        }
        $content .= "<li class=\"booktree\">" . $link. "</li>";
      }

      $ricursione++;
      foreach ($children as $child) {
        $leaf = \Drupal::entityTypeManager()
          ->getStorage('node')
          ->load($child->nid);

        $leaf = \Drupal::service('entity.repository')->getTranslationFromContext($leaf);
        $cache_tags[] = "node:$child->nid:$langcode";
        $c .= $this->booktree_mostra_figli($leaf, $node_start, $ricursione, $maxricursione, $trimval, $cache_tags, $link_to_pdf_sections);
      }

      if (strlen($c) > 2) {
        $content .= "<ul class=\"booktree\">\n" . $c . "</ul>\n";
      }

      return $content;
    }
    else {
      return '';
    }
  }
}
