<?php

namespace Drupal\book_pdfs\EventSubscriber;

use Drupal\book\BookExport;
use Drupal\book_pdfs\Event\BookPdfsEvents;
use Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Book PDFs event subscriber.
 */
class BookPdfsPreGenerateSubscriber implements EventSubscriberInterface {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The Book export service.
   *
   * @var \Drupal\book\BookExport
   */
  protected $bookExport;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The book_pdfs logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The book_pdfs settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * Constructs event subscriber.
   *
   * @param \Drupal\book\BookExport $bookExport
   *   The Book export service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(BookExport $bookExport, RendererInterface $renderer, LoggerChannelFactoryInterface $loggerFactory, RequestStack $requestStack, ConfigFactoryInterface $configFactory) {
    $this->bookExport = $bookExport;
    $this->renderer = $renderer;
    $this->logger = $loggerFactory->get('book_pdfs');
    $this->requestStack = $requestStack;
    $this->settings = $configFactory->get('book_pdfs.settings');
  }

  /**
   * Adds the main book body to the PDF being generated.
   *
   * @param \Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent $event
   *   The Book Pdf Pre Generate Event Object.
   *
   * @return \Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent|false
   *   Event Object.
   */
  public function addBookHtml(BookPdfsPreGenerateEvent $event) {
    $book = $event->getBook();
    $langcode = $event->getLangcode();
    if ($book->hasTranslation($langcode)) {
      $book = $book->getTranslation($langcode);
    }

    // Get the TOC mode setting.
    $toc_mode = $this->settings->get('toc_mode');

    // Disable this link_to_pdf idea, it's not needed, remove it later.
    $link_to_pdf = FALSE;

    try {
      // Generate table of contents if TOC mode is enabled.
      $tableOfContentsHtml = '';
      if ($toc_mode) {
        $tableOfContentsBuild = $this->generateTableOfContents($book, $link_to_pdf);
        $tableOfContentsHtml = $this->renderer->render($tableOfContentsBuild);
      }

      $bookBuild = $this->bookExport->bookExportHtml($book);
      $bookHtml = $this->renderer->render($bookBuild);

      // Add the table of contents at the beginning of the html contents if TOC mode is enabled.
      $bookHtmlWithTOC = $toc_mode ? $tableOfContentsHtml . $bookHtml : $bookHtml;
      $event->setHtml($event->getHtml() . $bookHtmlWithTOC);
    }
    catch (\Exception $e) {
      $this->logger->error('Error rendering Book export HTML. <br> %error', ['%error' => $e->getMessage()]);
      return FALSE;
    }

    return $event;
  }

  /**
   * Get the table of contents markup.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The Book node.
   * @param boolean $link_to_self_mode
   *   The mode.
   *
   * @return array
   *   The render array for the table of contents.
   */
  protected function generateTableOfContents($book, $link_to_self_mode) {
    $bookPdfController = \Drupal::classResolver()->getInstanceFromDefinition('\Drupal\book_pdfs\Controller\BooktreeController');
    $table_of_contents = $bookPdfController->tree($book, $link_to_self_mode);
    return $table_of_contents;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      BookPdfsEvents::PRE => ['addBookHtml'],
    ];
  }
}

